@@ -22,6 +22,7 @@
 
 /**
  * SECTION:gstiterator
+ * @title: GstIterator
  * @short_description: Object to retrieve multiple elements in a threadsafe
  * way.
  * @see_also: #GstElement, #GstBin
@@ -38,15 +39,15 @@
  * if it wants to continue using it later.
  *
  * The basic use pattern of an iterator is as follows:
- * |[
+ * |[<!-- language="C" -->
  *   GstIterator *it = _get_iterator(object);
  *   GValue item = G_VALUE_INIT;
  *   done = FALSE;
  *   while (!done) {
- *     switch (gst_iterator_next (it, &amp;item)) {
+ *     switch (gst_iterator_next (it, &item)) {
  *       case GST_ITERATOR_OK:
  *         ...get/use/change item here...
- *         g_value_reset (&amp;item);
+ *         g_value_reset (&item);
  *         break;
  *       case GST_ITERATOR_RESYNC:
  *         ...rollback changes to items...
@@ -61,12 +62,13 @@
  *         break;
  *     }
  *   }
- *   g_value_unset (&amp;item);
+ *   g_value_unset (&item);
  *   gst_iterator_free (it);
  * ]|
  */
 
 #include "gst_private.h"
+#include "glib-compat-private.h"
 #include <gst/gstiterator.h>
 
 /**
@@ -82,7 +84,7 @@ gst_iterator_copy (const GstIterator * it)
 {
   GstIterator *copy;
 
-  copy = g_slice_copy (it->size, it);
+  copy = g_memdup2 (it, it->size);
   if (it->copy)
     it->copy (it, copy);
 
@@ -158,7 +160,7 @@ gst_iterator_new (guint size,
   g_return_val_if_fail (resync != NULL, NULL);
   g_return_val_if_fail (free != NULL, NULL);
 
-  result = g_slice_alloc0 (size);
+  result = g_malloc0 (size);
   gst_iterator_init (result, size, type, lock, master_cookie, copy, next, item,
       resync, free);
 
@@ -227,7 +229,7 @@ gst_list_iterator_free (GstListIterator * it)
  * Create a new iterator designed for iterating @list.
  *
  * The list you iterate is usually part of a data structure @owner and is
- * protected with @lock. 
+ * protected with @lock.
  *
  * The iterator will use @lock to retrieve the next item of the list and it
  * will then call the @item function before releasing @lock again.
@@ -296,7 +298,7 @@ gst_iterator_pop (GstIterator * it)
  * @it: The #GstIterator to iterate
  * @elem: (out caller-allocates): pointer to hold next element
  *
- * Get the next item from the iterator in @elem. 
+ * Get the next item from the iterator in @elem.
  *
  * Only when this function returns %GST_ITERATOR_OK, @elem will contain a valid
  * value. @elem must have been initialized to the type of the iterator or
@@ -309,7 +311,7 @@ gst_iterator_pop (GstIterator * it)
  *
  * A return value of %GST_ITERATOR_RESYNC indicates that the element list was
  * concurrently updated. The user of @it should call gst_iterator_resync() to
- * get the newly updated list. 
+ * get the newly updated list.
  *
  * A return value of %GST_ITERATOR_ERROR indicates an unrecoverable fatal error.
  *
@@ -421,7 +423,7 @@ gst_iterator_free (GstIterator * it)
 
   it->free (it);
 
-  g_slice_free1 (it->size, it);
+  g_free (it);
 }
 
 /**
@@ -484,6 +486,7 @@ filter_next (GstIteratorFilter * it, GValue * elem)
         break;
       case GST_ITERATOR_RESYNC:
       case GST_ITERATOR_DONE:
+      case GST_ITERATOR_ERROR:
         done = TRUE;
         break;
       default:
@@ -607,6 +610,8 @@ gst_iterator_fold (GstIterator * it, GstIteratorFoldFunction func,
   GValue item = { 0, };
   GstIteratorResult result;
 
+  g_return_val_if_fail (it != NULL, GST_ITERATOR_ERROR);
+
   while (1) {
     result = gst_iterator_next (it, &item);
     switch (result) {
@@ -625,6 +630,7 @@ gst_iterator_fold (GstIterator * it, GstIteratorFoldFunction func,
   }
 
 fold_done:
+
   g_value_unset (&item);
 
   return result;
