@@ -27,10 +27,16 @@
  * objects and prints a list of leaks to the debug log under `GST_TRACER:7` when
  * gst_deinit() is called, and also prints a g_warning().
  *
- * Starting with GStreamer 1.18, you can also use action signals on the tracer
+ * Starting with GStreamer 1.18, you can also use GObject action signals on the tracer
  * object to fetch leak information. Use gst_tracing_get_active_tracers() to
  * get a list of all active tracers and find the right one by name.
  *
+ * If the `GST_LEAKS_TRACER_SIG` env variable is defined, you can use the
+ * following POSIX signals to interact with the leaks tracer:
+ * - SIGUSR1: log alive objects
+ * - SIGUSR2: create a checkpoint and print a list of objects created and
+ *   destroyed since the previous checkpoint.
+ *
  * You can activate this tracer in the usual way by adding the string 'leaks'
  * to the environment variable `GST_TRACERS`. Such as: `GST_TRACERS=leaks`
  *
@@ -726,6 +732,7 @@ process_leaks (GstLeaksTracer * self, GValue * ret_leaks)
 {
   GList *leaks, *l;
   gboolean ret = FALSE;
+  guint n = 0;
 
   if (!ret_leaks)
     GST_TRACE_OBJECT (self, "start listing currently alive objects");
@@ -737,8 +744,10 @@ process_leaks (GstLeaksTracer * self, GValue * ret_leaks)
     goto done;
   }
 
-  for (l = leaks; l; l = l->next)
+  for (l = leaks; l; l = l->next) {
     process_leak (l->data, ret_leaks);
+    n++;
+  }
 
   g_list_free_full (leaks, (GDestroyNotify) leak_free);
 
@@ -746,7 +755,7 @@ process_leaks (GstLeaksTracer * self, GValue * ret_leaks)
 
 done:
   if (!ret_leaks)
-    GST_TRACE_OBJECT (self, "done listing currently alive objects");
+    GST_TRACE_OBJECT (self, "listed %u alive objects", n);
 
   return ret;
 }
@@ -1036,6 +1045,7 @@ gst_leaks_tracer_activity_start_tracking (GstLeaksTracer * self)
   GST_OBJECT_LOCK (self);
   if (self->added) {
     GST_ERROR_OBJECT (self, "tracking is already in progress");
+    GST_OBJECT_UNLOCK (self);
     return;
   }
 
