@@ -161,6 +161,37 @@ GST_START_TEST (test_from_string)
   fail_unless_equals_int (g_value_get_boolean (val), TRUE);
   gst_structure_free (structure);
 
+  /* Test trailing comas */
+  s = "test-string,";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  gst_structure_free (structure);
+
+  s = "test-string,;";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  gst_structure_free (structure);
+
+  s = "test-string,value=true,";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (G_VALUE_HOLDS_BOOLEAN (val));
+  fail_unless_equals_int (g_value_get_boolean (val), TRUE);
+  gst_structure_free (structure);
+
+  s = "test-string,value=true,;";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (G_VALUE_HOLDS_BOOLEAN (val));
+  fail_unless_equals_int (g_value_get_boolean (val), TRUE);
+  gst_structure_free (structure);
+
+  s = "test-string,value=true,,";
+  structure = gst_structure_from_string (s, NULL);
+  fail_unless (structure == NULL, "Created structure from string %s", s);
+
   /* Tests for flagset deserialisation */
   s = "foobar,value=0010:ffff";
   structure = gst_structure_from_string (s, NULL);
@@ -188,9 +219,25 @@ GST_START_TEST (test_from_string)
   gst_structure_free (structure);
 
   s = "0.10:decoder-video/mpeg, abc=(boolean)false";
+  structure = NULL;
   ASSERT_CRITICAL (structure = gst_structure_from_string (s, NULL));
   fail_unless (structure == NULL, "Could not get structure from string %s", s);
 
+  /* Test that escaping works both with and without a type */
+  s = "foo/bar, value=\"raven \\\"nevermore\\\"\"";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (val));
+  gst_structure_free (structure);
+
+  s = "foo/bar, value=(string)\"raven \\\"nevermore\\\"\"";
+  structure = gst_structure_from_string (s, NULL);
+  fail_if (structure == NULL, "Could not get structure from string %s", s);
+  fail_unless ((val = gst_structure_get_value (structure, "value")) != NULL);
+  fail_unless (G_VALUE_HOLDS_STRING (val));
+  gst_structure_free (structure);
+
   /* make sure we bail out correctly in case of an error or if parsing fails */
   s = "***foo***, abc=(boolean)false";
   structure = gst_structure_from_string (s, NULL);
@@ -223,7 +270,7 @@ GST_END_TEST;
 
 GST_START_TEST (test_to_string)
 {
-  GstStructure *st1;
+  GstStructure *st1 = NULL;
 
   ASSERT_CRITICAL (st1 = gst_structure_new_empty ("Foo\nwith-newline"));
   fail_unless (st1 == NULL);
@@ -274,6 +321,20 @@ GST_START_TEST (test_to_from_string)
 
   gst_structure_free (st1);
   gst_structure_free (st2);
+
+  /* pointers are serialized but not deserialized */
+  st1 = gst_structure_new ("test", "ptr", G_TYPE_POINTER, 0xdeadbeef, NULL);
+  str = gst_structure_to_string (st1);
+  /* The way pointers are serialized may be plateform specific so just check
+   * if it contains the address */
+  fail_unless (g_strrstr (str, "deadbeef") || g_strrstr (str, "DEADBEEF"),
+      "Failed to serialize to right string: %s", str);
+
+  st2 = gst_structure_from_string (str, NULL);
+  fail_unless (!st2);
+
+  gst_structure_free (st1);
+  g_free (str);
 }
 
 GST_END_TEST;
@@ -363,7 +424,7 @@ GST_START_TEST (test_structure_new)
   GstStructure *s;
   GError *e;
   GQuark domain;
-  gboolean bool;
+  gboolean get_bool;
   gint num, den;
   GstClockTime clocktime;
   guint64 uint64;
@@ -387,8 +448,8 @@ GST_START_TEST (test_structure_new)
   fail_if (gst_structure_has_field (s, "key"));
   fail_unless_equals_int (gst_structure_n_fields (s), 4);
 
-  fail_unless (gst_structure_get_boolean (s, "bool", &bool));
-  fail_unless (bool);
+  fail_unless (gst_structure_get_boolean (s, "bool", &get_bool));
+  fail_unless (get_bool);
 
   fail_unless (gst_structure_get_fraction (s, "fraction", &num, &den));
   fail_unless_equals_int (num, 1);
@@ -412,6 +473,7 @@ GST_START_TEST (test_structure_new)
           ("0.10:decoder-video/mpeg")));
 
   /* make sure we bail out correctly in case of an error or if parsing fails */
+  s = NULL;
   ASSERT_CRITICAL (s = gst_structure_new ("^joo\nba\ndoo^",
           "abc", G_TYPE_BOOLEAN, FALSE, NULL));
   fail_unless (s == NULL);
@@ -422,17 +484,66 @@ GST_END_TEST;
 GST_START_TEST (test_fixate)
 {
   GstStructure *s;
+  gint i;
 
   s = gst_structure_new ("name",
       "int", G_TYPE_INT, 5,
-      "intrange", GST_TYPE_INT_RANGE, 5, 10,
-      "intrange2", GST_TYPE_INT_RANGE, 5, 10, NULL);
+      "intrange", GST_TYPE_INT_RANGE, 10, 20,
+      "intrange2", GST_TYPE_INT_RANGE, 10, 20,
+      "intrange3", GST_TYPE_INT_RANGE, 10, 20, NULL);
 
+  /* basic test */
   fail_if (gst_structure_fixate_field_nearest_int (s, "int", 5));
-  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange", 5));
-  fail_if (gst_structure_fixate_field_nearest_int (s, "intrange", 5));
-  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange2", 15));
-  fail_if (gst_structure_fixate_field_nearest_int (s, "intrange2", 15));
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange", 15));
+  fail_if (gst_structure_fixate_field_nearest_int (s, "intrange", 15));
+  fail_unless (gst_structure_get_int (s, "intrange", &i));
+  fail_unless_equals_int (i, 15);
+
+  /* test range min */
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange2", 5));
+  fail_unless (gst_structure_get_int (s, "intrange2", &i));
+  fail_unless_equals_int (i, 10);
+
+  /* test range max */
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange3", 25));
+  fail_unless (gst_structure_get_int (s, "intrange3", &i));
+  fail_unless_equals_int (i, 20);
+
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_fixate_stepped)
+{
+  GstStructure *s;
+  GValue v = G_VALUE_INIT;
+  gint i;
+
+  s = gst_structure_new_empty ("name");
+
+  g_value_init (&v, GST_TYPE_INT_RANGE);
+  gst_value_set_int_range_step (&v, 10, 20, 5);
+  gst_structure_set_value (s, "intrange1", &v);
+  gst_structure_set_value (s, "intrange2", &v);
+  gst_structure_set_value (s, "intrange3", &v);
+  g_value_unset (&v);
+
+  /* exact */
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange1", 15));
+  fail_unless (gst_structure_get_int (s, "intrange1", &i));
+  fail_unless_equals_int (i, 15);
+
+  /* round down */
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange2", 12));
+  fail_unless (gst_structure_get_int (s, "intrange2", &i));
+  fail_unless_equals_int (i, 10);
+
+  /* round up */
+  fail_unless (gst_structure_fixate_field_nearest_int (s, "intrange3", 13));
+  fail_unless (gst_structure_get_int (s, "intrange3", &i));
+  fail_unless_equals_int (i, 15);
+
   gst_structure_free (s);
 }
 
@@ -658,6 +769,41 @@ GST_START_TEST (test_structure_nested_from_and_to_string)
 
 GST_END_TEST;
 
+GST_START_TEST (test_serialize_nested_structures)
+{
+  GstStructure *s;
+  const gchar *str1;
+  gchar *str2, *end = NULL;
+
+  str1 = "main"
+      ", main-sub1=(structure)[type-b, machine-type=(int)0;]"
+      ", main-sub2=(structure)[type-a, plugin-filename=(string)\"/home/user/lib/lib\\ with\\ spaces.dll\", machine-type=(int)1;]"
+      ", main-sub3=(structure)[type-b, plugin-filename=(string)/home/user/lib/lib_no_spaces.so, machine-type=(int)1;]"
+      ", main-sub4=(structure){ [s1, a=(int)1;], [s2, b=(int)2;] }" ";";
+
+  s = gst_structure_from_string (str1, &end);
+  fail_unless (s != NULL);
+
+  GST_DEBUG ("not parsed part : %s", end);
+  fail_unless (*end == '\0');
+
+  fail_unless (gst_structure_n_fields (s) == 4);
+
+  fail_unless (gst_structure_has_field_typed (s, "main-sub1",
+          GST_TYPE_STRUCTURE));
+
+  str2 = gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_NONE);
+  fail_unless (str2 != NULL);
+
+  fail_unless_equals_string (str1, str2);
+
+  g_free (str2);
+
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_vararg_getters)
 {
   GstStructure *s;
@@ -872,6 +1018,41 @@ GST_START_TEST (test_flagset)
 
 GST_END_TEST;
 
+GST_START_TEST (test_flags)
+{
+  GstStructure *s;
+  GstSeekFlags flags = GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT;
+  guint parsed_flags = 0;
+
+  s = gst_structure_new ("test-struct", "test-seek-flags",
+      GST_TYPE_SEEK_FLAGS, flags, NULL);
+  fail_unless (gst_structure_get_flags (s, "test-seek-flags",
+          GST_TYPE_SEEK_FLAGS, &parsed_flags));
+  fail_unless (flags == (GstSeekFlags) parsed_flags);
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_strict)
+{
+  GstStructure *s;
+
+  GstElement *bin = gst_bin_new ("mybin");
+  s = gst_structure_new ("test-struct", "obj", GST_TYPE_BIN, bin, NULL);
+  fail_unless (s);
+  fail_if (gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_STRICT));
+  gst_structure_free (s);
+  gst_object_unref (bin);
+
+  s = gst_structure_new ("test-struct", "ptr", G_TYPE_POINTER, NULL, NULL);
+  fail_unless (s);
+  fail_if (gst_structure_serialize_full (s, GST_SERIALIZE_FLAG_STRICT));
+  gst_structure_free (s);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_structure_suite (void)
 {
@@ -889,6 +1070,7 @@ gst_structure_suite (void)
   tcase_add_test (tc_chain, test_complete_structure);
   tcase_add_test (tc_chain, test_structure_new);
   tcase_add_test (tc_chain, test_fixate);
+  tcase_add_test (tc_chain, test_fixate_stepped);
   tcase_add_test (tc_chain, test_fixate_frac_list);
   tcase_add_test (tc_chain, test_is_subset_equal_array_list);
   tcase_add_test (tc_chain, test_is_subset_different_name);
@@ -897,11 +1079,14 @@ gst_structure_suite (void)
   tcase_add_test (tc_chain, test_is_subset_superset_extra_values);
   tcase_add_test (tc_chain, test_structure_nested);
   tcase_add_test (tc_chain, test_structure_nested_from_and_to_string);
+  tcase_add_test (tc_chain, test_serialize_nested_structures);
   tcase_add_test (tc_chain, test_vararg_getters);
   tcase_add_test (tc_chain, test_foreach);
   tcase_add_test (tc_chain, test_map_in_place);
   tcase_add_test (tc_chain, test_filter_and_map_in_place);
   tcase_add_test (tc_chain, test_flagset);
+  tcase_add_test (tc_chain, test_flags);
+  tcase_add_test (tc_chain, test_strict);
   return s;
 }
 
