@@ -96,6 +96,14 @@ GST_START_TEST (basetransform_chain_pt1)
   gst_test_trans_push_segment (trans);
 
   gst_test_trans_free (trans);
+
+  klass_transform_ip = NULL;
+  klass_transform = NULL;
+  klass_transform_caps = NULL;
+  klass_transform_size = NULL;
+  klass_set_caps = NULL;
+  klass_submit_input_buffer = NULL;
+  klass_generate_output = NULL;
 }
 
 GST_END_TEST;
@@ -879,6 +887,80 @@ GST_START_TEST (basetransform_chain_ct3)
 
 GST_END_TEST;
 
+static gboolean transform_fixate_caps_invalid_called = FALSE;
+
+static GstCaps *
+transform_fixate_caps_invalid (G_GNUC_UNUSED GstBaseTransform * trans,
+    G_GNUC_UNUSED GstPadDirection direction, G_GNUC_UNUSED GstCaps * caps,
+    GstCaps * othercaps)
+{
+  transform_fixate_caps_invalid_called = TRUE;
+  gst_caps_unref (othercaps);
+  return NULL;
+}
+
+GST_START_TEST (basetransform_invalid_fixatecaps_impl)
+{
+  TestTransData *trans;
+  GstBuffer *buffer;
+  GstFlowReturn res;
+  GstCaps *caps;
+
+  klass_fixate_caps = transform_fixate_caps_invalid;
+
+  trans = gst_test_trans_new ();
+
+  gst_test_trans_push_segment (trans);
+
+  /* passthrough without caps */
+  buffer = gst_buffer_new_and_alloc (20);
+  res = gst_test_trans_push (trans, buffer);
+  fail_unless (res == GST_FLOW_OK);
+
+  buffer = gst_test_trans_pop (trans);
+  fail_unless (buffer != NULL);
+  fail_unless (gst_buffer_get_size (buffer) == 20);
+
+  /* try to set caps prior to push data
+   * all following attempts to fixate_caps would fail producing a g_critical */
+  gst_pad_push_event (trans->srcpad, gst_event_new_flush_start ());
+  gst_pad_push_event (trans->srcpad, gst_event_new_flush_stop (TRUE));
+
+  caps = gst_caps_from_string ("foo/x-bar");
+  ASSERT_CRITICAL (gst_test_trans_setcaps (trans, caps));
+  gst_caps_unref (caps);
+
+  ASSERT_CRITICAL (gst_test_trans_push_segment (trans));
+
+  ASSERT_CRITICAL (res = gst_test_trans_push (trans, buffer));
+  fail_unless (transform_fixate_caps_invalid_called == TRUE);
+  fail_unless (res == GST_FLOW_NOT_NEGOTIATED);
+
+  gst_test_trans_free (trans);
+}
+
+GST_END_TEST;
+
+static void
+transform1_setup (void)
+{
+  sink_template = &gst_test_trans_sink_template;
+  src_template = &gst_test_trans_src_template;
+}
+
+static void
+transform1_teardown (void)
+{
+  /* reset global state */
+  klass_transform_ip = NULL;
+  klass_transform = NULL;
+  klass_transform_caps = NULL;
+  klass_transform_size = NULL;
+  klass_set_caps = NULL;
+  klass_fixate_caps = NULL;
+  klass_submit_input_buffer = NULL;
+  klass_generate_output = NULL;
+}
 
 static Suite *
 gst_basetransform_suite (void)
@@ -887,6 +969,8 @@ gst_basetransform_suite (void)
   TCase *tc = tcase_create ("general");
 
   suite_add_tcase (s, tc);
+  tcase_add_checked_fixture (tc, transform1_setup, transform1_teardown);
+
   /* pass through */
   tcase_add_test (tc, basetransform_chain_pt1);
   tcase_add_test (tc, basetransform_chain_pt2);
@@ -898,6 +982,8 @@ gst_basetransform_suite (void)
   tcase_add_test (tc, basetransform_chain_ct2);
   tcase_add_test (tc, basetransform_chain_ct3);
 
+  tcase_add_test (tc, basetransform_invalid_fixatecaps_impl);
+
   return s;
 }
 
